@@ -4,7 +4,6 @@ from __future__ import absolute_import
 try:
     from . import waterfall
     from .waterfall import Waterfall
-    from blimpy.deprecated.filterbank import Filterbank, read_header, fix_header
     from .guppi import GuppiRaw
     from . import utils
 except ImportError:
@@ -14,6 +13,8 @@ try:
     from . import fil2h5
     from . import h52fil
     from . import bl_scrunch
+    from . import calcload
+    from . import rawhdr
 except:
     print("Warning: Cannot import HDF5 utilities")
 
