@@ -32,6 +32,7 @@ from .plotting import *
 from astropy.time import Time
 from astropy import units as u
 
+# YANKED 2021-03-05: from numba import jit
 
 #------
 # Logging set up
@@ -42,12 +43,12 @@ level_log = logging.INFO
 
 if level_log == logging.INFO:
     stream = sys.stdout
-    format = '%(name)-15s %(levelname)-8s %(message)s'
+    fmt = '%(name)-15s %(levelname)-8s %(message)s'
 else:
     stream =  sys.stderr
-    format = '%%(relativeCreated)5d (name)-15s %(levelname)-8s %(message)s'
+    fmt = '%%(relativeCreated)5d (name)-15s %(levelname)-8s %(message)s'
 
-logging.basicConfig(format=format,stream=stream,level = level_log)
+logging.basicConfig(format=fmt, stream=stream, level=level_log)
 
 
 #import pdb #pdb.set_trace()
@@ -59,14 +60,17 @@ MAX_BLOB_MB = 1024
 # Main blimpy class
 ###
 
-class Waterfall(object):
+class Waterfall():
     """ Class for loading and writing blimpy data (.fil, .h5) """
 
+    """ Get the frequency array for this Waterfall object"""    
+    get_freqs = lambda self: np.arange(0, self.header['nchans'], 1, dtype=float) * self.header['foff'] + self.header['fch1']
+
     def __repr__(self):
         return "Waterfall data: %s" % self.filename
 
     def __init__(self, filename=None, f_start=None, f_stop=None, t_start=None, t_stop=None,
-                 load_data=True, max_load=1., header_dict=None, data_array=None):
+                 load_data=True, max_load=None, header_dict=None, data_array=None):
         """ Class for loading and plotting blimpy data.
 
         This class parses the blimpy file and stores the header and data
@@ -82,8 +86,7 @@ class Waterfall(object):
             t_start (int): start integration ID
             t_stop (int): stop integration ID
             load_data (bool): load data. If set to False, only header will be read.
-            max_load (float): maximum data to load in GB. Default: 1GB.
-                              e.g. 0.1 is 100 MB
+            max_load (float): maximum data to load in GB.
             header_dict (dict): Create blimpy from header dictionary + data array
             data_array (np.array): Create blimpy from header dict + data array
         """
@@ -114,14 +117,14 @@ class Waterfall(object):
             self.__load_data()
 
         elif header_dict is not None and data_array is not None:
-            self.filename = b''
+            self.filename = ''
             self.header = header_dict
             self.data = data_array
             self.n_ints_in_file = 0
             self._setup_freqs()
 
         else:
-            self.filename = b''
+            self.filename = ''
 
         # Attach methods
         self.plot_spectrum         = six.create_bound_method(plot_spectrum, self)
@@ -130,7 +133,7 @@ class Waterfall(object):
         self.plot_time_series      = six.create_bound_method(plot_time_series, self)
         self.plot_all              = six.create_bound_method(plot_all, self)
         self.plot_spectrum_min_max = six.create_bound_method(plot_spectrum_min_max, self)
-
+    
     def __load_data(self):
         """ Helper for loading data from a container. Should not be called manually. """
 
@@ -156,16 +159,16 @@ class Waterfall(object):
         """ Updates the header information from the original file to the selection. """
 
         #Updating frequency of first channel from selection
-        if self.header[b'foff'] < 0:
-            self.header[b'fch1'] = self.container.f_stop
+        if self.header['foff'] < 0:
+            self.header['fch1'] = self.container.f_stop
         else:
-            self.header[b'fch1'] = self.container.f_start
+            self.header['fch1'] = self.container.f_start
 
         #Updating number of coarse channels.
-        self.header[b'nchans'] = self.container.selection_shape[self.freq_axis]
+        self.header['nchans'] = self.container.selection_shape[self.freq_axis]
 
         #Updating time stamp for first time bin from selection
-        self.header[b'tstart'] = self.container.populate_timestamps(update_header=True)
+        self.header['tstart'] = self.container.populate_timestamps(update_header=True)
 
     def info(self):
         """ Print header information and other derived information. """
@@ -179,7 +182,7 @@ class Waterfall(object):
                 val = val.to_string(unit=u.deg, sep=':')
             if key in ('foff', 'fch1'):
                 val *= u.MHz
-            if key == b'tstart':
+            if key == 'tstart':
                 print("%16s : %32s" % ("tstart (ISOT)", Time(val, format='mjd').isot))
                 key = "tstart (MJD)"
             print("%16s : %32s" % (key, val))
@@ -188,8 +191,14 @@ class Waterfall(object):
         print("%16s : %32s" % ("File shape", self.file_shape))
         print("--- Selection Info ---")
         print("%16s : %32s" % ("Data selection shape", self.selection_shape))
-        print("%16s : %32s" % ("Minimum freq (MHz)", self.container.f_start))
-        print("%16s : %32s" % ("Maximum freq (MHz)", self.container.f_stop))
+        if self.header['foff'] < 0:
+            minfreq = self.container.f_start - self.header['foff']
+            maxfreq = self.container.f_stop
+        else:
+            minfreq = self.container.f_start
+            maxfreq = self.container.f_stop - self.header['foff']
+        print("%16s : %32s" % ("Minimum freq (MHz)", minfreq))
+        print("%16s : %32s" % ("Maximum freq (MHz)", maxfreq))
 
 
     def _get_blob_dimensions(self, chunk_dim):
@@ -231,26 +240,27 @@ class Waterfall(object):
         """
 
         #Usually '.0000.' is in self.filename
-        if np.abs(self.header[b'foff']) < 1e-5:
+        if np.abs(self.header['foff']) < 1e-5:
             logger.info('Detecting high frequency resolution data.')
             chunk_dim = (1,1,1048576) #1048576 is the number of channels in a coarse channel.
             return chunk_dim
+
         #Usually '.0001.' is in self.filename
-        elif np.abs(self.header[b'tsamp']) < 1e-3:
+        if np.abs(self.header['tsamp']) < 1e-3:
             logger.info('Detecting high time resolution data.')
             chunk_dim = (2048,1,512) #512 is the total number of channels per single band (ie. blc00)
             return chunk_dim
+
         #Usually '.0002.' is in self.filename
-        elif np.abs(self.header[b'foff']) < 1e-2 and np.abs(self.header[b'foff'])  >= 1e-5:
+        if np.abs(self.header['foff']) < 1e-2 and np.abs(self.header['foff'])  >= 1e-5:
             logger.info('Detecting intermediate frequency and time resolution data.')
             chunk_dim = (10,1,65536)  #65536 is the total number of channels per single band (ie. blc00)
-#            chunk_dim = (1,1,65536/4)
-            return chunk_dim
-        else:
-            logger.warning('File format not known. Will use minimum chunking. NOT OPTIMAL.')
-            chunk_dim = (1,1,512)
             return chunk_dim
 
+        logger.warning('File format not known. Will use minimum chunking. NOT OPTIMAL.')
+        chunk_dim = (1,1,512)
+        return chunk_dim
+
     def calc_n_coarse_chan(self, chan_bw=None):
         """ This makes an attempt to calculate the number of coarse channels in a given freq selection.
 
@@ -277,23 +287,36 @@ class Waterfall(object):
             (freqs, data) (np.arrays): frequency axis in MHz and data subset
         """
 
-        self.freqs = self.container.populate_freqs()
-        self.timestamps = self.container.populate_timestamps()
+        if self.container.isheavy():
+            raise Exception("Waterfall.grab_data: Large data array was not loaded!  Try instantiating Waterfall with max_load.")
+
+        try:
+            self.freqs = self.container.populate_freqs()
+        except Exception as ex:
+            raise Exception("Waterfall.grab_data: Cannot allocate frequency array") from ex
+
+        try:
+            self.timestamps = self.container.populate_timestamps()
+        except Exception as ex:
+            raise Exception("Waterfall.grab_data: Cannot allocate timestamps array") from ex
 
         if f_start is None:
             f_start = self.freqs[0]
         if f_stop is None:
             f_stop = self.freqs[-1]
 
-        i0 = np.argmin(np.abs(self.freqs - f_start))
-        i1 = np.argmin(np.abs(self.freqs - f_stop))
+        try:
+            i0 = np.argmin(np.abs(self.freqs - f_start))
+            i1 = np.argmin(np.abs(self.freqs - f_stop))
 
-        if i0 < i1:
-            plot_f    = self.freqs[i0:i1 + 1]
-            plot_data = np.squeeze(self.data[t_start:t_stop, ..., i0:i1 + 1])
-        else:
-            plot_f    = self.freqs[i1:i0 + 1]
-            plot_data = np.squeeze(self.data[t_start:t_stop, ..., i1:i0 + 1])
+            if i0 < i1:
+                plot_f    = self.freqs[i0:i1 + 1]
+                plot_data = np.squeeze(self.data[t_start:t_stop, if_id, i0:i1 + 1])
+            else:
+                plot_f    = self.freqs[i1:i0 + 1]
+                plot_data = np.squeeze(self.data[t_start:t_stop, if_id, i1:i0 + 1])
+        except:
+            raise Exception("Waterfall.grab_data: Too much data requested")
 
         return plot_f, plot_data
 
@@ -328,9 +351,13 @@ class Waterfall(object):
 
         mid_chan = int(n_chan_per_coarse / 2)
 
-        for ii in range(n_coarse_chan):
-            ss = ii*n_chan_per_coarse
-            self.data[..., ss+mid_chan] = np.median(self.data[..., ss+mid_chan+5:ss+mid_chan+10])
+        # YANKED 2021-03-05: @jit(nopython=True, fastmath=True, cache=True)
+        def parse(data, n_coarse_chan, n_chan_per_coarse, mid_chan):
+            for ii in range(n_coarse_chan):
+                ss = ii*n_chan_per_coarse
+                data[..., ss+mid_chan] = np.median(data[..., ss+mid_chan+5:ss+mid_chan+10])
+        
+        parse(self.data, n_coarse_chan, n_chan_per_coarse, mid_chan)
 
     def calibrate_band_pass_N1(self):
         """ One way to calibrate the band pass is to take the median value
@@ -373,8 +400,8 @@ def cmd_tool(args=None):
                        help='save plot graphic to file (give filename as argument)')
     parser.add_argument('-S', action='store_true', default=False, dest='save_only',
                        help='Turn off plotting of data and only save to file.')
-    parser.add_argument('-D', action='store_false', default=True, dest='blank_dc',
-                       help='Use to not blank DC bin.')
+    parser.add_argument('-D', action='store_false', default=False, dest='blank_dc',
+                       help='Use to blank DC bin.')
     parser.add_argument('-H', action='store_true', default=False, dest='to_hdf5',
                        help='Write file to hdf5 format.')
     parser.add_argument('-F', action='store_true', default=False, dest='to_fil',
@@ -449,9 +476,12 @@ def cmd_tool(args=None):
         fileroot = os.path.splitext(filename)[0]
 
         if parse_args.to_hdf5 and parse_args.to_fil:
-            raise Warning('Either provide to_hdf5 or to_fil, but not both.')
+            # If this is the last statement anyway,
+            # and consequently, an output file is not generated,
+            # why do we not raise an error?
+            raise ValueError('Either provide to_hdf5 or to_fil, but not both.')
 
-        elif parse_args.to_hdf5:
+        if parse_args.to_hdf5:
             if not filename_out:
                 filename_out = fileroot + '.h5'
 
